;********************************
;*    Hicolor viewer for GBC    *
;********************************
;
;
; by Jeff Frohwein
;
; Started   4-Aug-99
; Last edit 4-Aug-99

RGBSet:         MACRO
        DW      ((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
        ENDM

        INCLUDE "addsub1.inc"
        INCLUDE "hardware.inc"

;Version 1.2 or later of 'hardware.inc' is required.
        rev_Check_hardware_inc 1.5

SpriteRoutine EQU $ff80

ChangeRAMBank EQU $cfff

        SECTION "Low Ram",BSS

;Reserved for color pallete code

SpriteAttrTab   ds      40*4

        SECTION "Org $0",HOME[$0]

Org00:
        di
        jr      Org00

        SECTION "Org $38",HOME[$38]
Org38:  di
        jr      Org38

        SECTION "Org $40",HOME[$40]
        jp      DoVblank

        SECTION "Org $48",HOME[$48]
        reti

        SECTION "Org $50",HOME[$50]
        reti
        SECTION "Org $58",HOME[$58]
        reti
        SECTION "Org $60",HOME[$60]
        reti

        SECTION "Org $66",HOME[$66]
Org68:  di
        jr      Org68

DoVblank:
        push    af
        call    SpriteRoutine
        pop     af
        reti

BankCodeFirst:
        ld      [de],a
BankCodeLast:

; DMA copy sprite data to sprite ram

VBlankFirst:
        ld      a,SpriteAttrTab/256
        ld      [rDMA],a

        ld      a,40
VBlnkW: dec     a
        jr      nz,VBlnkW

        ret
VBlankLast:

        DB      "Date 990804"           ; Date stamping info for
                                        ; reference use only

        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      begin

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "HICOLOR2",0,0,0,"0000",$80     ; Cart name   16bytes
 DB 0,0,0                     ; Not used
 DB CART_ROM                  ; Cart type
 DB CART_ROM_8M               ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1,1                       ; Maker ID
 DB 0                         ; Version     =0
 DB 0                         ; Complement check (important)
 DW 0                         ; Checksum (not important)

; Library Includes

        INCLUDE "memory1.asm"

        SECTION "Main Code",HOME

begin:
        di

        ld      sp,$ffff

        call    Initialize

        ld      a,1
        ld      [rSVBK],a       ; Set the first RAM bank

mainloop:
        ld      de,rSVBK
        ld      hl,rBCPS
        call    $d000

; Set BG palette 4 to grey scale

        ld      a,$a0
        ld      [hl+],a

        halt

        ld      [hl],$ff
        ld      [hl],$7f

        ld      [hl],$ce
        ld      [hl],$39

        ld      [hl],8
        ld      [hl],$21

        ld      [hl],0
        ld      [hl],0

        ld      a,1
        ld      [rSVBK],a       ; Set the first RAM bank

WaitForVBlank:
        halt
        ld      a,[rLY]
        cp      $90
        jr      nz,WaitForVBlank

; Don't start palette updates until the 5th screen line

        halt
        halt
        halt
        halt
        halt
        halt

        jr      mainloop


Initialize:

        ld      a,10
        ld      [rRAMG],a           ; Enable cart RAM

        ld      a,1
        ld      [rROMB0],a

        call    DoubleSpeed

        call    ScreenOff

        xor     a
        ld      [rSCX],a
        ld      a,0
        ld      [rSCY],a

; Install sprite DMA code

        ld      hl,VBlankFirst
        ld      de,SpriteRoutine
        ld      bc,VBlankLast-VBlankFirst
        call    mem_Copy

; Install change RAM bank code

        ld      hl,BankCodeFirst
        ld      de,ChangeRAMBank
        ld      bc,BankCodeLast-BankCodeFirst
        call    mem_Copy

        ld      a,$07
        ld      [rLCDC],a

; Fill $8000-9fff with 0 in both VRAM banks

        xor     a
        ld      [rVBK],a

        ld      hl,$8000
        ld      bc,$2000
        xor     a
        call    mem_Set

        ld      hl,SpriteTiles
        ld      de,$8020
        ld      bc,32
        call    mem_Copy

        ld      a,1
        ld      [rVBK],a

        ld      hl,$8000
        ld      bc,$2000
        xor     a
        call    mem_Set

; Fill attribute memory with color palette 4, bank 1

        ld      a,1
        ld      [rVBK],a

        ld      hl,$9800
        ld      bc,$400
        ld      a,$c
        call    mem_Set

; Setup color palette grid in attribute memory

        ld      hl,$9822
        ld      a,$80
        ld      de,$10
        ld      c,e
.loopg1:
        ld      b,8
        xor     a
.loopg2:
        ld      [hl+],a
        ld      [hl+],a
        inc     a
        dec     b
        jr      nz,.loopg2

        add     hl,de
        dec     c
        jr      nz,.loopg1

; Setup tile pattern grid in tile map memory

        xor     a
        ld      [rVBK],a

        ld      hl,$9822
        ld      a,$80
        ld      de,$10
        ld      c,e
.loopp1:
        ld      b,e
.loopp2:
        ld      [hl+],a
        inc     a
        dec     b
        jr      nz,.loopp2
        add     hl,de
        dec     c
        jr      nz,.loopp1

; Setup sprites to cover left & right borders of screen

        ld      hl,SpriteAttrTab
        ld      a,19
        ld      b,10

sprlp:  ld      [hl+],a
        ld      [hl],8
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        ld      [hl+],a
        ld      [hl],16
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        ld      [hl+],a
        ld      [hl],152
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        ld      [hl+],a
        ld      [hl],160
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        add     16
        dec     b
        jr      nz,sprlp

        ld      hl,rOCPS
        ld      [hl],$80

        inc     l

; Set all sprite palettes to white

        ld      b,32
.sprcl:
        ld      [hl],$ff
        ld      [hl],$7f
        dec     b
        jr      nz,.sprcl

; Copy tileset to VRAM & install color table

        ld      hl,$4000
        call    InstallTileSet

        ret


InstallTileSet:
        di
        xor     a
        ld      [rVBK],a

; Copy tile set to memory

        ld      de,$8800
        ld      bc,4096
        call    mem_Copy

; Install color table code

        ld      hl,$5000
        call    InstallColorTable

        ld      a,3
        ld      [rIE],a

        ld      a,8
        ld      [rSTAT],a

        xor     a
        ld      [rIF],a

        call    ScreenOn

.loop:  ld      a,[rLY]
        cp      $91
        jr      nz,.loop

        ei
        halt

        ret

; Install large color table
; Entry: DE = pntr to color data

InstallColorTable:
 if 1
        ld      d,h
        ld      e,l

        ld      hl,$d000

        ld      a,1
        ld      [rSVBK],a       ; Set the first RAM bank

        ld      c,49             ;129
        call    .loop

        ld      [hl],$3e        ; LD A,??
        inc     hl
        ld      [hl],2
        inc     hl

        ld      [hl],$c3        ; JP ChangeRAMBank
        inc     hl
        ld      [hl],(ChangeRAMBank%256)
        inc     hl
        ld      [hl],(ChangeRAMBank/256)

        ld      a,2
        ld      [rSVBK],a       ; Set the first RAM bank

        ld      hl,$d000

        ld      c,50
        call    .loop

        ld      [hl],$3e        ; LD A,??
        inc     hl
        ld      [hl],3
        inc     hl

        ld      [hl],$c3        ; JP ChangeRAMBank
        inc     hl
        ld      [hl],(ChangeRAMBank%256)
        inc     hl
        ld      [hl],(ChangeRAMBank/256)

        ld      a,3
        ld      [rSVBK],a       ; Set the first RAM bank

        ld      hl,$d000

        ld      c,30
        call    .loop

        ld      [hl],$c9        ; RET
        ret
 else
        ld      d,h
        ld      e,l

        ld      hl,$a0a0

        ld      c,129           ;129
        call    .loop

        ld      [hl],$c9        ; RET
        ret
 endc

.loop:  push    bc

        ld      [hl],$3e        ; LD A,??
        inc     hl
        ld      a,c
        inc     a
        rrca
        rrca
        rrca
        and     $20
        or      $80
        ld      [hl+],a

        ld      [hl],$22        ; LD [HL+],A
        inc     hl

        ld      [hl],$76        ; HALT
        inc     hl

        ld      b,16
        call    .loop2          ; LD [HL],?? x 32

        ld      [hl],$2d        ; DEC L
        inc     hl

        pop     bc
        dec     c
        jr      nz,.loop

        ret

.loop2:
        push    bc

        ld      a,[de]
        inc     de
        ld      c,a
        ld      a,[de]
        inc     de
        ld      b,a

        ld      a,$36
        ld      [hl+],a
        ld      [hl],c
        inc     hl
        ld      [hl+],a
        ld      [hl],b
        inc     hl
        pop     bc
        dec     b
        jr      nz,.loop2

        ret

; *** Turn screen on ***

ScreenOn:
        ld      hl,rLCDC
        set     7,[hl]
        ret

; *** Turn screen off ***

ScreenOff:
        ld      hl,rLCDC
        bit     7,[hl]          ; Is LCD already off?
        ret     z               ; yes, exit

        ld      a,[rIE]
        push    af
        res     0,a
        ld      [rIE],a         ; Disable vblank interrupt if enabled

.loop:  ld      a,[rLY]         ; Loop until in first part of vblank
        cp      145
        jr      nz,.loop

        res     7,[hl]          ; Turn the screen off

        pop     af
        ld      [rIE],a         ; Restore the state of vblank interrupt
        ret

; Set single speed CPU speed

SingleSpeed:
        ld      a,[rKEY1]
        rlca
        ret     nc
        jr      CPUToggleSpeed

; Set double speed CPU speed

DoubleSpeed:
        ld      a,[rKEY1]
        rlca
        ret     c

CPUToggleSpeed:
        di

        ld      hl,rIE
        ld      a,[hl]
        push    af

        xor     a
        ld      [hl],a         ;disable interrupts
        ld      [rIF],a

        ld      a,$30
        ld      [rP1],a

        ld      a,1
        ld      [rKEY1],a

        stop

        pop     af
        ld      [hl],a

        ei
        ret

SpriteTiles:
        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

